@@ -15,7 +15,7 @@ sub canonicalize {
 
   my $parts = $self->parts;
   for (my $i = 0; $i <= $#$parts;) {
-    if ($parts->[$i] eq '.' || $parts->[$i] eq '') { splice @$parts, $i, 1 }
+    if ($parts->[$i] =~ /^(?:\.||\.{3,})$/) { splice @$parts, $i, 1 }
     elsif ($i < 1 || $parts->[$i] ne '..' || $parts->[$i - 1] eq '..') { $i++ }
     else { splice @$parts, --$i, 2 }
   }
@@ -169,7 +169,9 @@ following new ones.
 
   $path = $path->canonicalize;
 
-Canonicalize path.
+Canonicalize path, parts comprised solely of three or more dots are treated as
+a single dot, to protect certain operating systems from path traversal
+attacks.
 
   # "/foo/baz"
   Mojo::Path->new('/foo/./bar/../baz')->canonicalize;
@@ -177,6 +179,12 @@ Canonicalize path.
   # "/../baz"
   Mojo::Path->new('/foo/../bar/../../baz')->canonicalize;
 
+  # "/foo/bar"
+  Mojo::Path->new('/foo/.../bar')->canonicalize;
+
+  # "/foo/bar"
+  Mojo::Path->new('/foo/...../bar')->canonicalize;
+
 =head2 clone
 
   my $clone = $path->clone;
