@@ -4,7 +4,6 @@ use Mojo::Base 'Mojo::Server::Daemon';
 use Fcntl ':flock';
 use File::Spec::Functions qw(catfile tmpdir);
 use IO::Poll qw(POLLIN POLLPRI);
-use List::Util 'shuffle';
 use Mojo::Util 'steady_time';
 use POSIX 'WNOHANG';
 use Scalar::Util 'weaken';
@@ -66,7 +65,8 @@ sub run {
   my $self = shift;
 
   # No Windows support
-  say 'Preforking not available for Windows.' and exit 0 if $^O eq 'MSWin32';
+  say 'Preforking is not available for Windows.' and exit 0
+    if $^O eq 'MSWin32';
 
   # Prepare lock file and event loop
   $self->{lock_file} = $self->lock_file . ".$$";
@@ -79,18 +79,19 @@ sub run {
   $self->{poll}->mask($self->{reader}, POLLIN | POLLPRI);
 
   # Clean manager environment
-  local $SIG{INT} = local $SIG{TERM} = sub { $self->_term };
   local $SIG{CHLD} = sub {
     while ((my $pid = waitpid -1, WNOHANG) > 0) {
       $self->emit(reap => $pid)->_stopped($pid);
     }
   };
+  local $SIG{INT} = local $SIG{TERM} = sub { $self->_term };
   local $SIG{QUIT} = sub { $self->_term(1) };
   local $SIG{TTIN} = sub { $self->workers($self->workers + 1) };
   local $SIG{TTOU} = sub {
-    $self->workers($self->workers - 1) if $self->workers > 0;
-    return unless $self->workers;
-    $self->{pool}{shuffle keys %{$self->{pool}}}{graceful} ||= steady_time;
+    $self->workers > 0 ? $self->workers($self->workers - 1) : return;
+    for my $w (values %{$self->{pool}}) {
+      ($w->{graceful} = steady_time) and last unless $w->{graceful};
+    }
   };
 
   # Preload application before starting workers
@@ -188,9 +189,10 @@ sub _spawn {
   $loop->recurring($self->heartbeat_interval => $cb);
 
   # Clean worker environment
-  $SIG{$_} = 'DEFAULT' for qw(INT TERM CHLD TTIN TTOU);
+  $SIG{$_} = 'DEFAULT' for qw(CHLD INT TERM TTIN TTOU);
   $SIG{QUIT} = sub { $loop->max_connections(0) };
   delete @$self{qw(poll reader)};
+  srand;
 
   $self->app->log->debug("Worker $$ started");
   $loop->start;
